% The MOSEK Optimization Toolbox For Matlab 
%
%
% General purpose
%   mosekopt  - Computational engine.
%
% Linear optimization
%   msklpopt  - Easy-to-use interface to mosekopt.
%   linprog   - MATLAB optimization toolbox compatible function.
%
% Quadratic optimization
%   mskqpopt  - Easy-to-use interface to mosekopt.
%   quadprog  - MATLAB optimization toolbox compatible function.
%  
% Linear least squares 
%   lsqlin    - Linear least squares with linear constraints.
%                 MATLAB optimization toolbox compatible function.
%   lsqnonneg - Linear least squares with nonnegativity constraints.
%                 MATLAB optimization toolbox compatible function.   
%
% Diagnostics function
%   mosekdiag   - Print out diagnostics information for a MOSEK installation.
%
% Controlling defaults and options.
%   mskoptimset - Used to create and modify the MATLAB optimization OPTIONS structure. 
%   mskoptimget - Used to obtain optimization parameters from the MATLAB OPTIONS structure. 
%
%% Copyright (c) MOSEK ApS, Denmark. All rights reserved.
